@@ -1,9 +1,11 @@
 import os
 import pathlib
+import hashlib
 from urllib.parse import urlparse
+from xdg import XDG_CACHE_HOME
 
 from oauth2client import client, tools, transport
-from oauth2client.file import Storage
+from .keyring_storage import Storage
 
 from . import context
 
@@ -12,22 +14,36 @@ TOKENS_FILENAME = "{}/tokens"
 
 
 def get_store_from_ctx(ctx, token=False):
-    return get_store(context.get_issuer(ctx), context.get_client_id(ctx), token=token)
+    return get_store(
+        issuer=context.get_issuer(ctx),
+        client_id=context.get_client_id(ctx),
+        scopes=context.get_scopes(ctx),
+        token=token,
+        admin=ctx.obj["ADMIN_MODE"],
+    )
 
 
-def get_store(issuer, client_id, token=False):
-    """ Return a oauth2client.file.Storage object """
-    home_dir = os.path.join(os.path.expanduser("~"), ".config", "agilicus")
+def get_store(issuer, client_id, scopes, token=False, admin=False):
+    """Return a oauth2client.file.Storage object"""
+    home_dir = os.path.join(XDG_CACHE_HOME, "agilicus-cli")
+    sha256 = hashlib.new("sha256")
     issuer_dir = urlparse(issuer).netloc
 
+    sha256.update(str(scopes).encode())
+
     if token:
         creds_file = TOKENS_FILENAME.format(client_id)
     else:
         creds_file = CREDS_FILENAME.format(client_id)
+    astr = admin and "a-" or ""
+    creds_file = f"{creds_file}-{astr}{sha256.hexdigest()[0:6]}"
 
     store_dir = os.path.join(home_dir, issuer_dir)
     store_file = os.path.join(store_dir, creds_file)
-    pathlib.Path(os.path.dirname(store_file)).mkdir(parents=True, exist_ok=True)
+    pathlib.Path(os.path.dirname(store_file)).mkdir(
+        mode=0o0700, parents=True, exist_ok=True
+    )
+    pathlib.Path(store_file).touch(mode=0o0600, exist_ok=True)
     return Storage(store_file)
 
 
@@ -36,11 +52,6 @@ def delete_credentials_with_ctx(ctx):
     store.delete()
 
 
-def delete_credentials(issuer, client_id):
-    store = get_store(issuer, client_id)
-    store.delete()
-
-
 def get_credentials_from_ctx(ctx, refresh=None):
     return get_credentials(
         context.get_issuer(ctx),
@@ -49,11 +60,18 @@ def get_credentials_from_ctx(ctx, refresh=None):
         context.get_scopes(ctx),
         context.get_auth_local_webserver(ctx),
         refresh=refresh,
+        admin=ctx.obj["ADMIN_MODE"],
     )
 
 
 def get_credentials(
-    issuer, client_id, cacert, agilicus_scopes, auth_local_webserver, refresh=None
+    issuer,
+    client_id,
+    cacert,
+    agilicus_scopes,
+    auth_local_webserver,
+    refresh=None,
+    admin=False,
 ):
     """Gets valid user credentials from storage.
 
@@ -64,7 +82,7 @@ def get_credentials(
         Credentials, the obtained credential.
     """
 
-    store = get_store(issuer, client_id)
+    store = get_store(issuer, client_id, agilicus_scopes, admin=admin)
     crd = store.get()
 
     # Configure the ouath2 flow to trust the api if a certificate is
@@ -89,8 +107,12 @@ def get_credentials(
             crd = None
 
     if crd and refresh:
-        crd.refresh(http)
-        return crd
+        try:
+            crd.refresh(http)
+            return crd
+        except Exception:
+            # on failure of refresh, force it to None and do reauth.
+            crd = None
 
     if not crd:
         client_info = {}
@@ -107,6 +129,7 @@ def get_credentials(
         ]
         if agilicus_scopes:
             scopes += agilicus_scopes
+        scopes = list(set(scopes))
         client_info["redirect_uri"] = ["http://localhost:4200"]
 
         constructor_kwargs = {
