@@ -1,3 +1,4 @@
+from agilicus.input_helpers import pop_item_if_none, strip_none
 from . import context
 
 
@@ -19,10 +20,10 @@ def query_top(
 
     apiclient = context.get_apiclient(ctx, token)
 
-    params = {}
+    pop_item_if_none(kwargs)
+    params = strip_none(kwargs)
     params["dt_from"] = dt_from
     params["dt_to"] = dt_to
-    params["sub_org_id"] = sub_org_id
 
     if app_id:
         params["app_id"] = app_id
@@ -53,10 +54,10 @@ def query_active(
 
     apiclient = context.get_apiclient(ctx, token)
 
-    params = {}
+    params = strip_none(kwargs)
+    pop_item_if_none(kwargs)
     params["dt_from"] = dt_from
     params["dt_to"] = dt_to
-    params["sub_org_id"] = sub_org_id
 
     if app_id:
         params["app_id"] = app_id
