@@ -8,7 +8,12 @@ class Token:
 
     def decode(self):
         if not self.dict:
-            self.dict = jwt.decode(self.token, verify=False)
+            self.dict = jwt.decode(
+                self.token,
+                algorithms=["ES256"],
+                options={"verify_signature": False},
+                leeway=60,
+            )
         return self.dict
 
     def hasRole(self, hasapp, hasrole):
